

import os


FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
REGION_DIR = os.path.join(FIXTURES_DIR, 'wof-region')
COUNTY_DIR = os.path.join(FIXTURES_DIR, 'wof-county')
LOCALITY_DIR = os.path.join(FIXTURES_DIR, 'wof-locality')
